#region License and copyright notice
/* 
 * Kaliko Content Management System
 * 
 * Copyright (c) Fredrik Schultz and Contributors
 * 
 * This source is subject to the Microsoft Public License.
 * See http://www.microsoft.com/opensource/licenses.mspx#Ms-PL.
 * All other rights reserved.
 * 
 * THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
 * EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
 */
#endregion

namespace KalikoCMS.DataStore {
    using System;
    using System.Web;
    using Configuration;
    using KalikoCMS.Core;
    using KalikoCMS.Data;
    using KalikoCMS.Serialization;
    using RaptorDB;

    public class RaptorDataStore : DataStore {
        private static readonly RaptorDBString RaptorDb = GetRaptorDatabase();

        public RaptorDataStore(CmsPage page)
            : base(page) {
        }

        public RaptorDataStore(Guid id)
            : base(id) {
        }

        public RaptorDataStore(string id)
            : base(id) {
        }

        public override T Get<T>(string objectName) {
            string key = CreateKey(objectName);
            string value;

            RaptorDb.Get(key, out value);

            T objectInstance = JsonSerialization.DeserializeJson<T>(value);

            return objectInstance;
        }

        public override void Store(string objectName, object instance) {
            string key = CreateKey(objectName);
            string value = JsonSerialization.SerializeJson(instance);

            RaptorDb.Set(key, value);
        }

        private static RaptorDBString GetRaptorDatabase() {
            string dataStorePath = GetDataStorePathFromConfig();
            string mappedPath = HttpContext.Current.Server.MapPath(dataStorePath);
            return new RaptorDBString(mappedPath, false);
        }

        private static string GetDataStorePathFromConfig() {
            string dataStorePath = SiteSettings.Instance.DataStorePath;

            if (string.IsNullOrEmpty(dataStorePath)) {
                return "/App_Data/DataStore";
            }

            return dataStorePath;
        }
    }
}